@@ -5,11 +5,11 @@ our $AUTHORITY = 'cpan:GENE';
 
 use v5.36;
 
-our $VERSION = '0.0203';
+our $VERSION = '0.0300';
 
 use Moo;
 use strictures 2;
-use Carp qw(croak carp);
+use Carp qw(croak);
 use Future::AsyncAwait;
 use IO::Async::Channel ();
 use IO::Async::Loop ();
@@ -20,10 +20,12 @@ use namespace::clean;
 
 
 has verbose => (
-    is      => 'ro',
-    isa     => sub { croak "$_[0] is not a boolean" unless $_[0] =~ /^[01]$/ },
-    default => sub { 0 },
+    is => 'lazy',
 );
+sub _build_verbose {
+    my ($self) = @_;
+    return $ENV{PERL_FUTURE_DEBUG} ? 1 : 0;
+}
 
 
 has input => (
@@ -69,6 +71,7 @@ has _midi_out => (
 
 sub BUILD {
     my ($self) = @_;
+
     my $midi_rtn = IO::Async::Routine->new(
         channels_in  => [ $self->_msg_channel ],
         channels_out => [ $self->_midi_channel ],
@@ -86,19 +89,21 @@ sub BUILD {
     $self->_msg_channel->send(\$input_name);
 
     $self->_midi_out->open_virtual_port('foo');
+
+    _log(sprintf 'Opening %s port %s...', $self->_midi_out->{type}, $self->output)
+        if $self->verbose;
     _open_port($self->_midi_out, $self->output);
+    _log(sprintf 'Opened %s port %s', $self->_midi_out->{type}, $self->output)
+        if $self->verbose;
 }
 
 sub _log {
-    return unless $ENV{PERL_FUTURE_DEBUG};
-    carp @_;
+    print join("\n", @_), "\n";
 }
 
 sub _open_port($device, $name) {
-    _log("Opening $device->{type} port $name ...");
     $device->open_port_by_name(qr/\Q$name/i)
         || croak "Failed to open port $name";
-    _log("Opened $device->{type} port $name");
 }
 
 sub _rtmidi_loop ($msg_ch, $midi_ch) {
@@ -118,11 +123,13 @@ sub _filter_and_forward ($self, $event) {
 
 
 sub send_it ($self, $event) {
+    _log("Event: @$event") if $self->verbose;
     $self->_midi_out->send_event($event->@*);
 }
 
 
 sub delay_send ($self, $delay_time, $event) {
+    _log("Event: @$event, Delay: $delay_time") if $self->verbose;
     $self->loop->add(
         IO::Async::Timer::Countdown->new(
             delay     => $delay_time,
@@ -137,7 +144,9 @@ sub run ($self) {
 }
 
 
-sub add_filter ($self, $event_type, $action) {
+sub add_filter ($self, $name, $event_type, $action) {
+    _log("Add $name filter for $event_type")
+        if $self->verbose;
     push $self->filters->{$event_type}->@*, $action;
 }
 
@@ -155,7 +164,7 @@ MIDI::RtController - Control your MIDI controller
 
 =head1 VERSION
 
-version 0.0203
+version 0.0300
 
 =head1 SYNOPSIS
 
@@ -178,7 +187,8 @@ version 0.0203
     return 0;
   }
 
-  $rtc->add_filter($_ => \&filter_tone) for qw(note_on note_off);
+  $rtc->add_filter('filter_tone', $_ => \&filter_tone)
+    for qw(note_on note_off);
 
   # add other stuff to the $rtc->loop...
 
@@ -251,9 +261,9 @@ Run the asynchronous B<loop>!
 
 =head2 add_filter
 
-  $rtc->add_filter($event_type => $action);
+  $rtc->add_filter($name, $event_type, $action);
 
-Add a filter, defined by the CODE reference B<action>, for an
+Add a named filter, defined by the CODE reference B<action>, for an
 B<event_type> like C<note_on> or C<note_off>.
 
 =head1 THANK YOU
